(*
 * Type and variable environments.
 *
 * ----------------------------------------------------------------
 *
 * @begin[license]
 * Copyright (C) 2000 Jason Hickey, Caltech
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * Author: Jason Hickey
 * @email{jyh@cs.caltech.edu}
 * @end[license]
 *)

open Fc_ir_type
open Fc_ir_exn_type

(*
 * Environments are abstract.
 *)
type tenv
type venv

(*
 * Global variables.
 *)
val global_vars : var list

(*
 * Type environment operations.
 *)
val empty_tenv : tenv
val tenv_add : tenv -> var -> ty -> tenv
val tenv_lookup : 'a raise_exn -> tenv -> 'a -> var -> ty
val tenv_expand : 'a raise_exn -> tenv -> 'a -> ty -> ty
val tenv_fold : ('a -> var -> ty list -> 'a) -> 'a -> tenv -> 'a

(*
 * Variable environment operations.
 *)
val empty_venv : venv
val venv_add : venv -> var -> ty -> venv
val venv_mem : venv -> var -> bool
val venv_lookup : 'a raise_exn -> venv -> 'a -> var -> ty
val venv_lookup_opt : venv -> var -> ty option
val venv_lookup_expand : 'a raise_exn -> tenv -> venv -> 'a -> var -> ty

(*!
 * @docoff
 *
 * -*-
 * Local Variables:
 * Caml-master: "compile"
 * End:
 * -*-
 *)
